unit PipeObjs;

interface

uses
  Windows, Classes, SysUtils;

type
  EPipe = class(Exception);

  TPipe = class(TObject)
    pipeHandle: THandle;
    rc        : Boolean;

    destructor  Destroy; override;

    procedure Read(var buf; bufsize: Integer; var BytesRead: Integer);
    procedure Write(var buf; bufsize: Integer; var BytesWritten: Integer);
    procedure ReadStr(var buf: AnsiString);
    procedure WriteStr(ToSend: AnsiString);
  end;

  TPipeServer = class(TPipe)
    constructor Create(AName: AnsiString; var sa: TSecurityAttributes); virtual;

    function Connect: Boolean;
  end;

  TPipeClient = class(TPipe)
    constructor Create(AName: AnsiString); virtual;
  end;

implementation

procedure TPipe.Read(var buf; bufsize: Integer; var BytesRead: Integer);
begin
  rc:= ReadFile(pipeHandle, buf, bufsize, BytesRead, nil);
  if not rc then
    raise EPipe.Create('TPipe.Read : Broken pipe, error = ' + IntToStr(GetLastError));
end;

procedure TPipe.Write(var buf; bufsize: Integer; var BytesWritten: Integer);
begin
  rc:= WriteFile(pipeHandle, buf, bufsize, BytesWritten, nil);
  if not rc then
    raise EPipe.Create('TPipe.Write : Broken pipe, error = ' + IntToStr(GetLastError));
end;

procedure TPipe.ReadStr(var buf: AnsiString);
var
  BytesRead: Integer;
begin
  SetLength(buf, 255);
  Read(buf[1], length(buf), BytesRead);
  SetLength(buf, BytesRead);
end;

procedure TPipe.WriteStr(ToSend: AnsiString);
var
  buf: AnsiString;
  BytesWritten: Integer;
begin
  buf:= ToSend;
  Write(buf[1], length(buf), BytesWritten);
end;


destructor TPipe.Destroy;
begin
  if pipeHandle <> 0 then CloseHandle(pipeHandle);
  Inherited Destroy;
end;

constructor TPipeServer.Create(AName: AnsiString; var sa: TSecurityAttributes);
begin
  Inherited Create;
  pipeHandle:= CreateNamedPipe(PChar(AName), PIPE_ACCESS_DUPLEX, PIPE_TYPE_MESSAGE +
                               PIPE_WAIT, 1, 0, 0, 150, @sa);
  if pipeHandle = INVALID_HANDLE_VALUE then
    raise EPipe.Create('TPipeServer.Create, error = ' + IntToStr(GetLastError));
end;

function TPipeServer.Connect: Boolean;
begin
  Result:= ConnectNamedPipe(pipeHandle, nil);
end;

constructor TPipeClient.Create(AName: AnsiString);
begin
  Inherited Create;
  pipeHandle:= CreateFile(PChar(AName), GENERIC_READ + GENERIC_WRITE,
                          FILE_SHARE_READ, nil, OPEN_EXISTING,
                          FILE_ATTRIBUTE_NORMAL, 0);
  if pipeHandle = INVALID_HANDLE_VALUE then
    raise EPipe.Create('TPipeClient.Create, error = ' + IntToStr(GetLastError));
end;

end.
